@@ -1,12 +1,10 @@
 # -*- test-case-name: twisted.test.test_defer -*-
 #
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
-"""Support for results that aren't immediately available.
-
-API Stability: stable
+"""
+Support for results that aren't immediately available.
 
 Maintainer: U{Glyph Lefkowitz<mailto:glyph@twistedmatrix.com>}
 """
@@ -16,7 +14,7 @@ import traceback
 import warnings
 
 # Twisted imports
-from twisted.python import log, failure
+from twisted.python import log, failure, lockfile
 from twisted.python.util import unsignedID, mergeFunctionMetadata
 
 class AlreadyCalledError(Exception):
@@ -50,24 +48,25 @@ def succeed(result):
     d.callback(result)
     return d
 
-class _nothing: pass
 
-def fail(result=_nothing):
+def fail(result=None):
     """
     Return a Deferred that has already had '.errback(result)' called.
 
     See L{succeed}'s docstring for rationale.
 
-    @param result: The same argument that L{Deferred.errback<twisted.internet.defer.Deferred.errback>} takes.
+    @param result: The same argument that L{Deferred.errback} takes.
+
+    @raise NoCurrentExceptionError: If C{result} is C{None} but there is no
+        current exception state.
 
     @rtype: L{Deferred}
     """
-    if result is _nothing:
-        result = failure.Failure()
     d = Deferred()
     d.errback(result)
     return d
 
+
 def execute(callable, *args, **kw):
     """Create a deferred from a callable and arguments.
 
@@ -155,6 +154,11 @@ class Deferred:
     timeoutCall = None
     _debugInfo = None
 
+    # Are we currently running a user-installed callback?  Meant to prevent
+    # recursive running of callbacks when a reentrant call to add a callback is
+    # used.
+    _runningCallbacks = False
+
     # Keep this class attribute for now, for compatibility with code that
     # sets it directly.
     debug = False
@@ -177,7 +181,7 @@ class Deferred:
         cbs = ((callback, callbackArgs, callbackKeywords),
                (errback or (passthru), errbackArgs, errbackKeywords))
         self.callbacks.append(cbs)
-            
+
         if self.called:
             self._runCallbacks()
         return self
@@ -213,7 +217,17 @@ class Deferred:
         """Chain another Deferred to this Deferred.
 
         This method adds callbacks to this Deferred to call d's callback or
-        errback, as appropriate."""
+        errback, as appropriate. It is merely a shorthand way of performing
+        the following::
+
+            self.addCallbacks(d.callback, d.errback)
+
+        When you chain a deferred d2 to another deferred d1 with
+        d1.chainDeferred(d2), you are making d2 participate in the callback
+        chain of d1. Thus any event that fires d1 will also fire d2.
+        However, the converse is B{not} true; if d2 is fired d1 will not be
+        affected.
+        """
         return self.addCallbacks(d.callback, d.errback)
 
     def callback(self, result):
@@ -230,7 +244,8 @@ class Deferred:
 
 
     def errback(self, fail=None):
-        """Run all error callbacks that have been added to this Deferred.
+        """
+        Run all error callbacks that have been added to this Deferred.
 
         Each callback will have its result passed as the first
         argument to the next; this way, the callbacks act as a
@@ -244,6 +259,9 @@ class Deferred:
 
         Passing a string as `fail' is deprecated, and will be punished with
         a warning message.
+
+        @raise NoCurrentExceptionError: If C{fail} is C{None} but there is
+            no current exception state.
         """
         if not isinstance(fail, failure.Failure):
             fail = failure.Failure(fail)
@@ -294,26 +312,31 @@ class Deferred:
         self._runCallbacks()
 
     def _runCallbacks(self):
+        if self._runningCallbacks:
+            # Don't recursively run callbacks
+            return
         if not self.paused:
-            cb = self.callbacks
-            self.callbacks = []
-            while cb:
-                item = cb.pop(0)
+            while self.callbacks:
+                item = self.callbacks.pop(0)
                 callback, args, kw = item[
                     isinstance(self.result, failure.Failure)]
                 args = args or ()
                 kw = kw or {}
                 try:
-                    self.result = callback(self.result, *args, **kw)
+                    self._runningCallbacks = True
+                    try:
+                        self.result = callback(self.result, *args, **kw)
+                    finally:
+                        self._runningCallbacks = False
                     if isinstance(self.result, Deferred):
-                        self.callbacks = cb
-
                         # note: this will cause _runCallbacks to be called
-                        # "recursively" sometimes... this shouldn't cause any
-                        # problems, since all the state has been set back to
-                        # the way it's supposed to be, but it is useful to know
-                        # in case something goes wrong.  deferreds really ought
-                        # not to return themselves from their callbacks.
+                        # recursively if self.result already has a result.
+                        # This shouldn't cause any problems, since there is no
+                        # relevant state in this stack frame at this point.
+                        # The recursive call will continue to process
+                        # self.callbacks until it is empty, then return here,
+                        # where there is no more work to be done, so this call
+                        # will return as well.
                         self.pause()
                         self.result.addBoth(self._continue)
                         break
@@ -393,7 +416,7 @@ class DebugInfo:
 
 class FirstError(Exception):
     """First error to occur in a DeferredList if fireOnOneErrback is set.
-    
+
     @ivar subFailure: the L{Failure} that occurred.
     @ivar index: the index of the Deferred in the DeferredList where it
     happened.
@@ -424,7 +447,7 @@ class FirstError(Exception):
         if isinstance(other, tuple):
             return tuple(self) == other
         elif isinstance(other, FirstError):
-            return (self.subFailure == other.subFailure and 
+            return (self.subFailure == other.subFailure and
                     self.index == other.index)
         return False
 
@@ -527,16 +550,90 @@ FAILURE = False
 
 
 
+## deferredGenerator
 
 class waitForDeferred:
     """
-    API Stability: semi-stable
+    See L{deferredGenerator}.
+    """
 
+    def __init__(self, d):
+        if not isinstance(d, Deferred):
+            raise TypeError("You must give waitForDeferred a Deferred. You gave it %r." % (d,))
+        self.d = d
+
+
+    def getResult(self):
+        if isinstance(self.result, failure.Failure):
+            self.result.raiseException()
+        return self.result
+
+
+
+def _deferGenerator(g, deferred):
+    """
+    See L{deferredGenerator}.
+    """
+    result = None
+
+    # This function is complicated by the need to prevent unbounded recursion
+    # arising from repeatedly yielding immediately ready deferreds.  This while
+    # loop and the waiting variable solve that by manually unfolding the
+    # recursion.
+
+    waiting = [True, # defgen is waiting for result?
+               None] # result
+
+    while 1:
+        try:
+            result = g.next()
+        except StopIteration:
+            deferred.callback(result)
+            return deferred
+        except:
+            deferred.errback()
+            return deferred
+
+        # Deferred.callback(Deferred) raises an error; we catch this case
+        # early here and give a nicer error message to the user in case
+        # they yield a Deferred.
+        if isinstance(result, Deferred):
+            return fail(TypeError("Yield waitForDeferred(d), not d!"))
+
+        if isinstance(result, waitForDeferred):
+            # a waitForDeferred was yielded, get the result.
+            # Pass result in so it don't get changed going around the loop
+            # This isn't a problem for waiting, as it's only reused if
+            # gotResult has already been executed.
+            def gotResult(r, result=result):
+                result.result = r
+                if waiting[0]:
+                    waiting[0] = False
+                    waiting[1] = r
+                else:
+                    _deferGenerator(g, deferred)
+            result.d.addBoth(gotResult)
+            if waiting[0]:
+                # Haven't called back yet, set flag so that we get reinvoked
+                # and return from the loop
+                waiting[0] = False
+                return deferred
+            # Reset waiting to initial values for next loop
+            waiting[0] = True
+            waiting[1] = None
+
+            result = None
+
+
+
+def deferredGenerator(f):
+    """
     Maintainer: U{Christopher Armstrong<mailto:radix@twistedmatrix.com>}
 
-    waitForDeferred and deferredGenerator help you write Deferred-using code
-    that looks like it's blocking (but isn't really), with the help of
-    generators.
+    deferredGenerator and waitForDeferred help you write Deferred-using code
+    that looks like a regular sequential function. If your code has a minimum
+    requirement of Python 2.5, consider the use of L{inlineCallbacks} instead,
+    which can accomplish the same thing in a more concise manner.
 
     There are two important functions involved: waitForDeferred, and
     deferredGenerator.  They are used together, like this::
@@ -587,73 +684,147 @@ class waitForDeferred:
     Deferred to the 'blocking' style, and deferredGenerator converts from the
     'blocking' style to a Deferred.
     """
-
-    def __init__(self, d):
-        if not isinstance(d, Deferred):
-            raise TypeError("You must give waitForDeferred a Deferred. You gave it %r." % (d,))
-        self.d = d
+    def unwindGenerator(*args, **kwargs):
+        return _deferGenerator(f(*args, **kwargs), Deferred())
+    return mergeFunctionMetadata(f, unwindGenerator)
 
 
-    def getResult(self):
-        if isinstance(self.result, failure.Failure):
-            self.result.raiseException()
-        return self.result
+## inlineCallbacks
 
+# BaseException is only in Py 2.5.
+try:
+    BaseException
+except NameError:
+    BaseException=Exception
 
+class _DefGen_Return(BaseException):
+    def __init__(self, value):
+        self.value = value
 
-def _deferGenerator(g, deferred=None):
+def returnValue(val):
     """
-    See L{waitForDeferred}.
+    Return val from a L{inlineCallbacks} generator.
+
+    Note: this is currently implemented by raising an exception
+    derived from BaseException.  You might want to change any
+    'except:' clauses to an 'except Exception:' clause so as not to
+    catch this exception.
+
+    Also: while this function currently will work when called from
+    within arbitrary functions called from within the generator, do
+    not rely upon this behavior.
     """
-    result = None
+    raise _DefGen_Return(val)
+
+def _inlineCallbacks(result, g, deferred):
+    """
+    See L{inlineCallbacks}.
+    """
+    # This function is complicated by the need to prevent unbounded recursion
+    # arising from repeatedly yielding immediately ready deferreds.  This while
+    # loop and the waiting variable solve that by manually unfolding the
+    # recursion.
+
+    waiting = [True, # waiting for result?
+               None] # result
+
     while 1:
-        if deferred is None:
-            deferred = Deferred()
         try:
-            result = g.next()
+            # Send the last result back as the result of the yield expression.
+            if isinstance(result, failure.Failure):
+                result = result.throwExceptionIntoGenerator(g)
+            else:
+                result = g.send(result)
         except StopIteration:
-            deferred.callback(result)
+            # fell off the end, or "return" statement
+            deferred.callback(None)
+            return deferred
+        except _DefGen_Return, e:
+            # returnValue call
+            deferred.callback(e.value)
             return deferred
         except:
             deferred.errback()
             return deferred
 
-        # Deferred.callback(Deferred) raises an error; we catch this case
-        # early here and give a nicer error message to the user in case
-        # they yield a Deferred. Perhaps eventually these semantics may
-        # change.
         if isinstance(result, Deferred):
-            return fail(TypeError("Yield waitForDeferred(d), not d!"))
-
-        if isinstance(result, waitForDeferred):
-            waiting = [True, None]
-            # Pass vars in so they don't get changed going around the loop
-            def gotResult(r, waiting=waiting, result=result):
-                result.result = r
+            # a deferred was yielded, get the result.
+            def gotResult(r):
                 if waiting[0]:
                     waiting[0] = False
                     waiting[1] = r
                 else:
-                    _deferGenerator(g, deferred)
-            result.d.addBoth(gotResult)
+                    _inlineCallbacks(r, g, deferred)
+
+            result.addBoth(gotResult)
             if waiting[0]:
                 # Haven't called back yet, set flag so that we get reinvoked
                 # and return from the loop
                 waiting[0] = False
                 return deferred
-            result = None # waiting[1]
+
+            result = waiting[1]
+            # Reset waiting to initial values for next loop.  gotResult uses
+            # waiting, but this isn't a problem because gotResult is only
+            # executed once, and if it hasn't been executed yet, the return
+            # branch above would have been taken.
 
 
+            waiting[0] = True
+            waiting[1] = None
 
-def deferredGenerator(f):
+
+    return deferred
+
+def inlineCallbacks(f):
     """
-    See L{waitForDeferred}.
+    Maintainer: U{Christopher Armstrong<mailto:radix@twistedmatrix.com>}
+
+    WARNING: this function will not work in Python 2.4 and earlier!
+
+    inlineCallbacks helps you write Deferred-using code that looks like a
+    regular sequential function. This function uses features of Python 2.5
+    generators.  If you need to be compatible with Python 2.4 or before, use
+    the L{deferredGenerator} function instead, which accomplishes the same
+    thing, but with somewhat more boilerplate.  For example::
+
+        def thingummy():
+            thing = yield makeSomeRequestResultingInDeferred()
+            print thing #the result! hoorj!
+        thingummy = inlineCallbacks(thingummy)
+
+    When you call anything that results in a Deferred, you can simply yield it;
+    your generator will automatically be resumed when the Deferred's result is
+    available. The generator will be sent the result of the Deferred with the
+    'send' method on generators, or if the result was a failure, 'throw'.
+
+    Your inlineCallbacks-enabled generator will return a Deferred object, which
+    will result in the return value of the generator (or will fail with a
+    failure object if your generator raises an unhandled exception). Note that
+    you can't use 'return result' to return a value; use 'returnValue(result)'
+    instead. Falling off the end of the generator, or simply using 'return'
+    will cause the Deferred to have a result of None.
+
+    The Deferred returned from your deferred generator may errback if your
+    generator raised an exception::
+
+        def thingummy():
+            thing = yield makeSomeRequestResultingInDeferred()
+            if thing == 'I love Twisted':
+                # will become the result of the Deferred
+                returnValue('TWISTED IS GREAT!')
+            else:
+                # will trigger an errback
+                raise Exception('DESTROY ALL LIFE')
+        thingummy = inlineCallbacks(thingummy)
     """
     def unwindGenerator(*args, **kwargs):
-        return _deferGenerator(f(*args, **kwargs))
+        return _inlineCallbacks(None, f(*args, **kwargs), Deferred())
     return mergeFunctionMetadata(f, unwindGenerator)
 
 
+## DeferredLock/DeferredQueue
+
 class _ConcurrencyPrimitive(object):
     def __init__(self):
         self.waiting = []
@@ -694,9 +865,8 @@ class _ConcurrencyPrimitive(object):
 
 
 class DeferredLock(_ConcurrencyPrimitive):
-    """A lock for event driven systems.
-
-    API stability: Unstable
+    """
+    A lock for event driven systems.
 
     @ivar locked: True when this Lock has been acquired, false at all
     other times.  Do not change this value, but it is useful to
@@ -733,9 +903,8 @@ class DeferredLock(_ConcurrencyPrimitive):
             d.callback(self)
 
 class DeferredSemaphore(_ConcurrencyPrimitive):
-    """A semaphore for event driven systems.
-
-    API stability: Unstable
+    """
+    A semaphore for event driven systems.
     """
 
     def __init__(self, tokens):
@@ -779,9 +948,8 @@ class QueueUnderflow(Exception):
 
 
 class DeferredQueue(object):
-    """An event driven queue.
-
-    API stability: Unstable
+    """
+    An event driven queue.
 
     Objects may be added as usual to this queue.  When an attempt is
     made to retrieve an object when the queue is empty, a Deferred is
@@ -833,8 +1001,107 @@ class DeferredQueue(object):
             raise QueueUnderflow()
 
 
+class AlreadyTryingToLockError(Exception):
+    """
+    Raised when DeferredFilesystemLock.deferUntilLocked is called twice on a
+    single DeferredFilesystemLock.
+    """
+
+
+class DeferredFilesystemLock(lockfile.FilesystemLock):
+    """
+    A FilesystemLock that allows for a deferred to be fired when the lock is
+    acquired.
+
+    @ivar _scheduler: The object in charge of scheduling retries. In this
+        implementation this is parameterized for testing.
+
+    @ivar _interval: The retry interval for an L{IReactorTime} based scheduler.
+
+    @ivar _tryLockCall: A L{DelayedCall} based on _interval that will managex
+        the next retry for aquiring the lock.
+
+    @ivar _timeoutCall: A L{DelayedCall} based on deferUntilLocked's timeout
+        argument.  This is in charge of timing out our attempt to acquire the
+        lock.
+    """
+    _interval = 1
+    _tryLockCall = None
+    _timeoutCall = None
+
+    def __init__(self, name, scheduler=None):
+        """
+        @param name: The name of the lock to acquire
+        @param scheduler: An object which provides L{IReactorTime}
+        """
+        lockfile.FilesystemLock.__init__(self, name)
+
+        if scheduler is None:
+            from twisted.internet import reactor
+            scheduler = reactor
+
+        self._scheduler = scheduler
+
+    def deferUntilLocked(self, timeout=None):
+        """
+        Wait until we acquire this lock.  This method is not safe for
+        concurrent use.
+
+        @type timeout: C{float} or C{int}
+        @param timeout: the number of seconds after which to time out if the
+            lock has not been acquired.
+
+        @return: a deferred which will callback when the lock is acquired, or
+            errback with a L{TimeoutError} after timing out or an
+            L{AlreadyTryingToLockError} if the L{deferUntilLocked} has already
+            been called and not successfully locked the file.
+        """
+        if self._tryLockCall is not None:
+            return fail(
+                AlreadyTryingToLockError(
+                    "deferUntilLocked isn't safe for concurrent use."))
+
+        d = Deferred()
+
+        def _cancelLock():
+            self._tryLockCall.cancel()
+            self._tryLockCall = None
+            self._timeoutCall = None
+
+            if self.lock():
+                d.callback(None)
+            else:
+                d.errback(failure.Failure(
+                        TimeoutError("Timed out aquiring lock: %s after %fs" % (
+                                self.name,
+                                timeout))))
+
+        def _tryLock():
+            if self.lock():
+                if self._timeoutCall is not None:
+                    self._timeoutCall.cancel()
+                    self._timeoutCall = None
+
+                self._tryLockCall = None
+
+                d.callback(None)
+            else:
+                if timeout is not None and self._timeoutCall is None:
+                    self._timeoutCall = self._scheduler.callLater(
+                        timeout, _cancelLock)
+
+                self._tryLockCall = self._scheduler.callLater(
+                    self._interval, _tryLock)
+
+        _tryLock()
+
+        return d
+
+
 __all__ = ["Deferred", "DeferredList", "succeed", "fail", "FAILURE", "SUCCESS",
            "AlreadyCalledError", "TimeoutError", "gatherResults",
-           "maybeDeferred", "waitForDeferred", "deferredGenerator",
+           "maybeDeferred",
+           "waitForDeferred", "deferredGenerator", "inlineCallbacks",
            "DeferredLock", "DeferredSemaphore", "DeferredQueue",
+           "DeferredFilesystemLock", "AlreadyTryingToLockError",
           ]
