@@ -1,17 +1,19 @@
-# -*- test-case-name: twisted.test.test_util -*-
-
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# -*- test-case-name: twisted.python.test.test_util -*-
+# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
-from __future__ import nested_scopes, generators
-
-__version__ = '$Revision: 1.51 $'[11:-2]
-
 import os, sys, hmac, errno, new, inspect
-
+try:
+    import pwd, grp
+except ImportError:
+    pwd = grp = None
+try:
+    from os import setgroups, getgroups
+except ImportError:
+    setgroups = getgroups = None
 from UserDict import UserDict
 
+
 class InsensitiveDict:
     """Dictionary, that has case-insensitive keys.
 
@@ -540,31 +542,41 @@ class FancyStrMixin:
         return ''.join(r)
     __repr__ = __str__
 
+
+
 class FancyEqMixin:
     compareAttributes = ()
     def __eq__(self, other):
         if not self.compareAttributes:
             return self is other
-        #XXX Maybe get rid of this, and rather use hasattr()s
-        if not isinstance(other, self.__class__):
-            return False
-        for attr in self.compareAttributes:
-            if not getattr(self, attr) == getattr(other, attr):
-                return False
-        return True
+        if isinstance(self, other.__class__):
+            return (
+                [getattr(self, name) for name in self.compareAttributes] ==
+                [getattr(other, name) for name in self.compareAttributes])
+        return NotImplemented
+
 
     def __ne__(self, other):
-        return not self.__eq__(other)
+        result = self.__eq__(other)
+        if result is NotImplemented:
+            return result
+        return not result
+
+
 
 def dsu(list, key):
     L2 = [(key(e), i, e) for (i, e) in zip(range(len(list)), list)]
     L2.sort()
     return [e[2] for e in L2]
 
-try:
-    import pwd, grp
-    from os import setgroups, getgroups
-    
+if pwd is None or grp is None or setgroups is None or getgroups is None:
+    def initgroups(uid, primaryGid):
+        """
+        Do nothing.
+
+        Underlying platform support require to manipulate groups is missing.
+        """
+else:
     def _setgroups_until_success(l):
         while(1):
             # NASTY NASTY HACK (but glibc does it so it must be okay):
@@ -589,9 +601,10 @@ try:
             else:
                 # Success, yay!
                 return
-            
+
     def initgroups(uid, primaryGid):
-        """Initializes the group access list.
+        """
+        Initializes the group access list.
 
         This is done by reading the group database /etc/group and using all
         groups of which C{uid} is a member.  The additional group
@@ -600,14 +613,21 @@ try:
         If the given user is a member of more than C{NGROUPS}, arbitrary
         groups will be silently discarded to bring the number below that
         limit.
-        """       
+
+        @type uid: C{int}
+        @param uid: The UID for which to look up group information.
+
+        @type primaryGid: C{int} or C{NoneType}
+        @param primaryGid: If provided, an additional GID to include when
+            setting the groups.
+        """
         try:
             # Try to get the maximum number of groups
             max_groups = os.sysconf("SC_NGROUPS_MAX")
         except:
             # No predefined limit
             max_groups = 0
-        
+
         username = pwd.getpwuid(uid)[0]
         l = []
         if primaryGid is not None:
@@ -628,14 +648,7 @@ try:
                         raise
             else:
                 raise
-                                    
 
-except:
-    def initgroups(uid, primaryGid):
-        """Do nothing.
-
-        Underlying platform support require to manipulate groups is missing.
-        """
 
 
 def switchUID(uid, gid, euid=False):
@@ -763,28 +776,133 @@ def unsignedID(obj):
 
 def mergeFunctionMetadata(f, g):
     """
-    Overwrite C{g}'s docstring and name with values from C{f}.  Update
+    Overwrite C{g}'s name and docstring with values from C{f}.  Update
     C{g}'s instance dictionary with C{f}'s.
+
+    To use this function safely you must use the return value. In Python 2.3,
+    L{mergeFunctionMetadata} will create a new function. In later versions of
+    Python, C{g} will be mutated and returned.
+
+    @return: A function that has C{g}'s behavior and metadata merged from
+        C{f}.
     """
-    try:
-        g.__doc__ = f.__doc__
-    except (TypeError, AttributeError):
-        pass
-    try:
-        g.__dict__.update(f.__dict__)
-    except (TypeError, AttributeError):
-        pass
     try:
         g.__name__ = f.__name__
     except TypeError:
         try:
-            g = new.function(
+            merged = new.function(
                 g.func_code, g.func_globals,
                 f.__name__, inspect.getargspec(g)[-1],
                 g.func_closure)
         except TypeError:
             pass
-    return g
+    else:
+        merged = g
+    try:
+        merged.__doc__ = f.__doc__
+    except (TypeError, AttributeError):
+        pass
+    try:
+        merged.__dict__.update(g.__dict__)
+        merged.__dict__.update(f.__dict__)
+    except (TypeError, AttributeError):
+        pass
+    merged.__module__ = f.__module__
+    return merged
+
+
+def nameToLabel(mname):
+    """
+    Convert a string like a variable name into a slightly more human-friendly
+    string with spaces and capitalized letters.
+
+    @type mname: C{str}
+    @param mname: The name to convert to a label.  This must be a string
+    which could be used as a Python identifier.  Strings which do not take
+    this form will result in unpredictable behavior.
+
+    @rtype: C{str}
+    """
+    labelList = []
+    word = ''
+    lastWasUpper = False
+    for letter in mname:
+        if letter.isupper() == lastWasUpper:
+            # Continuing a word.
+            word += letter
+        else:
+            # breaking a word OR beginning a word
+            if lastWasUpper:
+                # could be either
+                if len(word) == 1:
+                    # keep going
+                    word += letter
+                else:
+                    # acronym
+                    # we're processing the lowercase letter after the acronym-then-capital
+                    lastWord = word[:-1]
+                    firstLetter = word[-1]
+                    labelList.append(lastWord)
+                    word = firstLetter + letter
+            else:
+                # definitely breaking: lower to upper
+                labelList.append(word)
+                word = letter
+        lastWasUpper = letter.isupper()
+    if labelList:
+        labelList[0] = labelList[0].capitalize()
+    else:
+        return mname.capitalize()
+    labelList.append(word)
+    return ' '.join(labelList)
+
+
+
+def uidFromString(uidString):
+    """
+    Convert a user identifier, as a string, into an integer UID.
+
+    @type uid: C{str}
+    @param uid: A string giving the base-ten representation of a UID or the
+        name of a user which can be converted to a UID via L{pwd.getpwnam}.
+
+    @rtype: C{int}
+    @return: The integer UID corresponding to the given string.
+
+    @raise ValueError: If the user name is supplied and L{pwd} is not
+        available.
+    """
+    try:
+        return int(uidString)
+    except ValueError:
+        if pwd is None:
+            raise
+        return pwd.getpwnam(uidString)[2]
+
+
+
+def gidFromString(gidString):
+    """
+    Convert a group identifier, as a string, into an integer GID.
+
+    @type uid: C{str}
+    @param uid: A string giving the base-ten representation of a GID or the
+        name of a group which can be converted to a GID via L{grp.getgrnam}.
+
+    @rtype: C{int}
+    @return: The integer GID corresponding to the given string.
+
+    @raise ValueError: If the group name is supplied and L{grp} is not
+        available.
+    """
+    try:
+        return int(gidString)
+    except ValueError:
+        if grp is None:
+            raise
+        return grp.getgrnam(gidString)[2]
+
+
 
 __all__ = [
     "uniquify", "padTo", "getPluginDirs", "addPluginDir", "sibpath",
@@ -792,5 +910,6 @@ __all__ = [
     "OrderedDict", "InsensitiveDict", "spewer", "searchupwards", "LineLog",
     "raises", "IntervalDifferential", "FancyStrMixin", "FancyEqMixin",
     "dsu", "switchUID", "SubclassableCStringIO", "moduleMovedForSplit",
-    "unsignedID", "mergeFunctionMetadata",
+    "unsignedID", "mergeFunctionMetadata", "nameToLabel", "uidFromString",
+    "gidFromString",
 ]
