@@ -1,21 +1,33 @@
 
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 
-
 """
 Test cases for defer module.
 """
 
-from __future__ import nested_scopes
+import gc
 
 from twisted.trial import unittest, util
 from twisted.internet import reactor, defer
 from twisted.python import failure, log
 
+from twisted.internet.task import Clock
+
+class GenericError(Exception):
+    pass
+
+
+_setTimeoutSuppression = util.suppress(
+    message="Deferred.setTimeout is deprecated.  Look for timeout "
+            "support specific to the API you are using instead.",
+    category=DeprecationWarning)
+
+_firstErrorSuppression = util.suppress(
+    message="FirstError.__getitem__ is deprecated.  Use attributes instead.",
+    category=DeprecationWarning)
 
-class GenericError(Exception): pass
 
 class DeferredTestCase(unittest.TestCase):
 
@@ -110,7 +122,7 @@ class DeferredTestCase(unittest.TestCase):
         dl = defer.DeferredList([], fireOnOneCallback=1)
         dl.addCallbacks(cb)
         self.failUnlessEqual(result, [])
-        
+
     def testDeferredListFireOnOneError(self):
         defr1 = defer.Deferred()
         defr2 = defer.Deferred()
@@ -137,15 +149,12 @@ class DeferredTestCase(unittest.TestCase):
         failure = result[0]
 
         # the type of the failure is a FirstError
-        self.failUnless(issubclass(failure.type, defer.FirstError), 
+        self.failUnless(issubclass(failure.type, defer.FirstError),
             'issubclass(failure.type, defer.FirstError) failed: '
             'failure.type is %r' % (failure.type,)
         )
 
-        # FirstErrors act like tuples for backwards compatibility :(
         firstError = failure.value
-        self.failUnlessEqual(firstError[0], firstError.subFailure)
-        self.failUnlessEqual(firstError[1], firstError.index)
 
         # check that the GenericError("2") from the deferred at index 1
         # (defr2) is intact inside failure.value
@@ -153,7 +162,20 @@ class DeferredTestCase(unittest.TestCase):
         self.failUnlessEqual(firstError.subFailure.value.args, ("from def2",))
         self.failUnlessEqual(firstError.index, 1)
 
-        
+
+    def test_indexingFirstError(self):
+        """
+        L{FirstError} behaves a little like a tuple, for backwards
+        compatibility.  Test that it can actually be indexed to retrieve
+        information about the failure.
+        """
+        subFailure = object()
+        index = object()
+        firstError = defer.FirstError(subFailure, index)
+        self.assertIdentical(firstError[0], firstError.subFailure)
+        self.assertIdentical(firstError[1], firstError.index)
+    test_indexingFirstError.suppress = [_firstErrorSuppression]
+
 
     def testDeferredListDontConsumeErrors(self):
         d1 = defer.Deferred()
@@ -217,17 +239,20 @@ class DeferredTestCase(unittest.TestCase):
         d1.addErrback(lambda e: None)  # Swallow error
 
     def testTimeOut(self):
+        """
+        Test that a Deferred which has setTimeout called on it and never has
+        C{callback} or C{errback} called on it eventually fails with a
+        L{error.TimeoutError}.
+        """
+        L = []
         d = defer.Deferred()
-        d.setTimeout(1.0)
-        l = []
-        d.addErrback(l.append)
-        # Make sure the reactor is shutdown
-        d.addBoth(lambda x, r=reactor: r.crash())
+        d.setTimeout(0.01)
+        self.assertFailure(d, defer.TimeoutError)
+        d.addCallback(L.append)
+        self.failIf(L, "Deferred failed too soon.")
+        return d
+    testTimeOut.suppress = [_setTimeoutSuppression]
 
-        self.assertEquals(l, [])
-        reactor.run()
-        self.assertEquals(len(l), 1)
-        self.assertEquals(l[0].type, defer.TimeoutError)
 
     def testImmediateSuccess(self):
         l = []
@@ -235,12 +260,19 @@ class DeferredTestCase(unittest.TestCase):
         d.addCallback(l.append)
         self.assertEquals(l, ["success"])
 
-    def testImmediateSuccess2(self):
+
+    def test_immediateSuccessBeforeTimeout(self):
+        """
+        Test that a synchronously successful Deferred is not affected by a
+        C{setTimeout} call.
+        """
         l = []
         d = defer.succeed("success")
         d.setTimeout(1.0)
         d.addCallback(l.append)
         self.assertEquals(l, ["success"])
+    test_immediateSuccessBeforeTimeout.suppress = [_setTimeoutSuppression]
+
 
     def testImmediateFailure(self):
         l = []
@@ -301,13 +333,25 @@ class DeferredTestCase(unittest.TestCase):
         # get rid of error
         dl[1].addErrback(lambda e: 1)
 
-    def testMaybeDeferred(self):
+
+    def test_maybeDeferredSync(self):
+        """
+        L{defer.maybeDeferred} should retrieve the result of a synchronous
+        function and pass it to its resulting L{defer.Deferred}.
+        """
         S, E = [], []
         d = defer.maybeDeferred((lambda x: x + 5), 10)
         d.addCallbacks(S.append, E.append)
         self.assertEquals(E, [])
         self.assertEquals(S, [15])
+        return d
 
+
+    def test_maybeDeferredSyncError(self):
+        """
+        L{defer.maybeDeferred} should catch exception raised by a synchronous
+        function and errback its resulting L{defer.Deferred} with it.
+        """
         S, E = [], []
         try:
             '10' + 5
@@ -318,18 +362,90 @@ class DeferredTestCase(unittest.TestCase):
         self.assertEquals(S, [])
         self.assertEquals(len(E), 1)
         self.assertEquals(str(E[0].value), expected)
-
-        d = defer.Deferred()
-        reactor.callLater(0.2, d.callback, 'Success')
-        d.addCallback(self.assertEquals, 'Success')
-        d.addCallback(self._testMaybeError)
         return d
 
-    def _testMaybeError(self, ignored):
+
+    def test_maybeDeferredAsync(self):
+        """
+        L{defer.maybeDeferred} should let L{defer.Deferred} instance pass by
+        so that original result is the same.
+        """
         d = defer.Deferred()
-        reactor.callLater(0.2, d.errback, failure.Failure(RuntimeError()))
-        self.assertFailure(d, RuntimeError)
-        return d
+        d2 = defer.maybeDeferred(lambda: d)
+        d.callback('Success')
+        return d2.addCallback(self.assertEquals, 'Success')
+
+
+    def test_maybeDeferredAsyncError(self):
+        """
+        L{defer.maybeDeferred} should let L{defer.Deferred} instance pass by
+        so that L{failure.Failure} returned by the original instance is the
+        same.
+        """
+        d = defer.Deferred()
+        d2 = defer.maybeDeferred(lambda: d)
+        d.errback(failure.Failure(RuntimeError()))
+        return self.assertFailure(d2, RuntimeError)
+
+
+    def test_reentrantRunCallbacks(self):
+        """
+        A callback added to a L{Deferred} by a callback on that L{Deferred}
+        should be added to the end of the callback chain.
+        """
+        deferred = defer.Deferred()
+        called = []
+        def callback3(result):
+            called.append(3)
+        def callback2(result):
+            called.append(2)
+        def callback1(result):
+            called.append(1)
+            deferred.addCallback(callback3)
+        deferred.addCallback(callback1)
+        deferred.addCallback(callback2)
+        deferred.callback(None)
+        self.assertEqual(called, [1, 2, 3])
+
+
+    def test_nonReentrantCallbacks(self):
+        """
+        A callback added to a L{Deferred} by a callback on that L{Deferred}
+        should not be executed until the running callback returns.
+        """
+        deferred = defer.Deferred()
+        called = []
+        def callback2(result):
+            called.append(2)
+        def callback1(result):
+            called.append(1)
+            deferred.addCallback(callback2)
+            self.assertEquals(called, [1])
+        deferred.addCallback(callback1)
+        deferred.callback(None)
+        self.assertEqual(called, [1, 2])
+
+
+    def test_reentrantRunCallbacksWithFailure(self):
+        """
+        After an exception is raised by a callback which was added to a
+        L{Deferred} by a callback on that L{Deferred}, the L{Deferred} should
+        call the first errback with a L{Failure} wrapping that exception.
+        """
+        exceptionMessage = "callback raised exception"
+        deferred = defer.Deferred()
+        def callback2(result):
+            raise Exception(exceptionMessage)
+        def callback1(result):
+            deferred.addCallback(callback2)
+        deferred.addCallback(callback1)
+        deferred.callback(None)
+        self.assertFailure(deferred, Exception)
+        def cbFailed(exception):
+            self.assertEqual(exception.args, (exceptionMessage,))
+        deferred.addCallback(cbFailed)
+        return deferred
+
 
 
 class AlreadyCalledTestCase(unittest.TestCase):
@@ -474,27 +590,69 @@ class AlreadyCalledTestCase(unittest.TestCase):
         d.addBoth(lambda ign: None)
 
 
+
 class LogTestCase(unittest.TestCase):
+    """
+    Test logging of unhandled errors.
+    """
 
     def setUp(self):
+        """
+        Add a custom observer to observer logging.
+        """
         self.c = []
         log.addObserver(self.c.append)
 
     def tearDown(self):
+        """
+        Remove the observer.
+        """
         log.removeObserver(self.c.append)
 
-    def testErrorLog(self):
-        c = self.c
-        defer.Deferred().addCallback(lambda x: 1/0).callback(1)
-# do you think it is rad to have memory leaks glyph
-##        d = defer.Deferred()
-##        d.addCallback(lambda x: 1/0)
-##        d.callback(1)
-##        del d
-        c2 = [e for e in c if e["isError"]]
+    def _check(self):
+        """
+        Check the output of the log observer to see if the error is present.
+        """
+        c2 = [e for e in self.c if e["isError"]]
         self.assertEquals(len(c2), 2)
         c2[1]["failure"].trap(ZeroDivisionError)
-        log.flushErrors(ZeroDivisionError)
+        self.flushLoggedErrors(ZeroDivisionError)
+
+    def test_errorLog(self):
+        """
+        Verify that when a Deferred with no references to it is fired, and its
+        final result (the one not handled by any callback) is an exception,
+        that exception will be logged immediately.
+        """
+        defer.Deferred().addCallback(lambda x: 1/0).callback(1)
+        self._check()
+
+    def test_errorLogWithInnerFrameRef(self):
+        """
+        Same as L{test_errorLog}, but with an inner frame.
+        """
+        def _subErrorLogWithInnerFrameRef():
+            d = defer.Deferred()
+            d.addCallback(lambda x: 1/0)
+            d.callback(1)
+
+        _subErrorLogWithInnerFrameRef()
+        gc.collect()
+        self._check()
+
+    def test_errorLogWithInnerFrameCycle(self):
+        """
+        Same as L{test_errorLogWithInnerFrameRef}, plus create a cycle.
+        """
+        def _subErrorLogWithInnerFrameCycle():
+            d = defer.Deferred()
+            d.addCallback(lambda x, d=d: 1/0)
+            d._d = d
+            d.callback(1)
+
+        _subErrorLogWithInnerFrameCycle()
+        gc.collect()
+        self._check()
 
 
 class DeferredTestCaseII(unittest.TestCase):
@@ -638,3 +796,104 @@ class OtherPrimitives(unittest.TestCase):
         queue = defer.DeferredQueue(backlog=0)
         self.assertRaises(defer.QueueUnderflow, queue.get)
 
+
+
+class DeferredFilesystemLockTestCase(unittest.TestCase):
+    """
+    Test the behavior of L{DeferredFilesystemLock}
+    """
+    def setUp(self):
+        self.clock = Clock()
+        self.lock = defer.DeferredFilesystemLock(self.mktemp(),
+                                                 scheduler=self.clock)
+
+
+    def test_waitUntilLockedWithNoLock(self):
+        """
+        Test that the lock can be acquired when no lock is held
+        """
+        d = self.lock.deferUntilLocked(timeout=1)
+
+        return d
+
+
+    def test_waitUntilLockedWithTimeoutLocked(self):
+        """
+        Test that the lock can not be acquired when the lock is held
+        for longer than the timeout.
+        """
+        self.failUnless(self.lock.lock())
+
+        d = self.lock.deferUntilLocked(timeout=5.5)
+        self.assertFailure(d, defer.TimeoutError)
+
+        self.clock.pump([1]*10)
+
+        return d
+
+
+    def test_waitUntilLockedWithTimeoutUnlocked(self):
+        """
+        Test that a lock can be acquired while a lock is held
+        but the lock is unlocked before our timeout.
+        """
+        def onTimeout(f):
+            f.trap(defer.TimeoutError)
+            self.fail("Should not have timed out")
+
+        self.failUnless(self.lock.lock())
+
+        self.clock.callLater(1, self.lock.unlock)
+        d = self.lock.deferUntilLocked(timeout=10)
+        d.addErrback(onTimeout)
+
+        self.clock.pump([1]*10)
+
+        return d
+
+
+    def test_defaultScheduler(self):
+        """
+        Test that the default scheduler is set up properly.
+        """
+        lock = defer.DeferredFilesystemLock(self.mktemp())
+
+        self.assertEquals(lock._scheduler, reactor)
+
+
+    def test_concurrentUsage(self):
+        """
+        Test that an appropriate exception is raised when attempting
+        to use deferUntilLocked concurrently.
+        """
+        self.lock.lock()
+        self.clock.callLater(1, self.lock.unlock)
+
+        d = self.lock.deferUntilLocked()
+        d2 = self.lock.deferUntilLocked()
+
+        self.assertFailure(d2, defer.AlreadyTryingToLockError)
+
+        self.clock.advance(1)
+
+        return d
+
+
+    def test_multipleUsages(self):
+        """
+        Test that a DeferredFilesystemLock can be used multiple times
+        """
+        def lockAquired(ign):
+            self.lock.unlock()
+            d = self.lock.deferUntilLocked()
+            return d
+
+        self.lock.lock()
+        self.clock.callLater(1, self.lock.unlock)
+
+        d = self.lock.deferUntilLocked()
+        d.addCallback(lockAquired)
+
+        self.clock.advance(1)
+
+        return d
